local addon, private = ...
local core = LibStub("AceAddon-3.0"):GetAddon("addon")
local L = LibStub("AceLocale-3.0"):GetLocale("addon")
local strformat, pairs, tinsert, unpack, strsub  = string.format, pairs, tinsert, unpack, string.sub
local GetItemIcon = GetItemIcon
local LTT = LibStub:GetLibrary("LibQTip-1.0")
local E, L, V, P, G = unpack(ElvUI)
local S = E:GetModule('Skins')
local tooltip

local classColors = {}
for class, color in pairs(CUSTOM_CLASS_COLORS or RAID_CLASS_COLORS) do
    classColors[class] = strformat( "%2x%2x%2x", color.r * 255, color.g * 255, color.b * 255 )
end

if CUSTOM_CLASS_COLORS then
    CUSTOM_CLASS_COLORS:RegisterCallback(function()
        for class, color in pairs(CUSTOM_CLASS_COLORS) do
            classColors[class] = strformat("%2x%2x%2x", color.r * 255, color.g * 255, color.b * 255)
        end
        end)
end

local myFont = CreateFont("ToolTipFont")
myFont:SetFont(GameTooltipText:GetFont(), 12)

local bcm = LibStub("LibDataBroker-1.1"):NewDataObject("Broker CrystalMote",{
    type  = "data source",
    text  = " ",
    label = "Broker CrystalMote",
    icon  = "Interface\\AddOns\\"..addon.."\\icon",
    OnClick = function(self, button)
        if button == "RightButton" then
            if InterfaceOptionsFrame:IsShown() then
                HideUIPanel(InterfaceOptionsFrame)
            end
            ToggleDropDownMenu(1, nil, BCMDropDownMenu, "cursor")
        elseif
            button == "LeftButton" then
            InterfaceOptionsFrame_OpenToCategory(core.optFrames.core)
        end
    end,
    OnEnter = function(self)
        if LTT:IsAcquired("BCMToolTip") then
            tooltip:Hide()
            tooltip:Clear()
        else
            tooltip = LTT:Acquire("BCMToolTip", 2, "LEFT", "RIGHT")
        end
        core:TooltipDraw()
        tooltip:SmartAnchorTo(self:GetParent())
        tooltip:SetAutoHideDelay( .1, self )
        tooltip:EnableMouse()
        tooltip:SetScrollStep(db.scroll - 25)
        tooltip:UpdateScrolling(db.scroll)
        if IsAddOnLoaded("ElvUI") and E.private["tooltip"].enable == true then
            tooltip:SetScript("OnShow", function(ttskinself) ttskinself:SetTemplate("Transparent") end)
        end
        tooltip:Show()
    end,
    OnLeave = function(self)
    end,
} )

private.bcm = bcm

function core:onTextUpdate()
    local tracktext = "BCM:"
    local textcount = 0
    for k,itemid in pairs (private.bcitems.id) do
        local iconpath = GetItemIcon(itemid)
        local icon = "|T"..iconpath..":12:12:0:0:64:64:4:60:4:60|t"
        local color = private.bcitems.color[k]
        local name = private.bcitems.strings[k]
        local shortname = private.bcitems.short[k]
        local text = core:createText(itemid, iconpath, icon, color, name, shortname, item)
        if text ~= nil then
            textcount = textcount + 1
            if textcount == 1 then
                tracktext = tracktext.." "..(text).." "..core:Color(chardb.data[itemid].inv)
            else
                tracktext = tracktext.."   "..(text).." "..core:Color(chardb.data[itemid].inv)
            end
        end
    end
    for k,itemid in pairs (private.lkitems.id) do
        local iconpath = GetItemIcon(itemid)
        local icon = "|T"..iconpath..":12:12:0:0:64:64:4:60:4:60|t"
        local color = private.lkitems.color[k]
        local name = private.lkitems.strings[k]
        local shortname = private.lkitems.short[k]
        local text = core:createText(itemid, iconpath, icon, color, name, shortname, item)
        if text ~= nil then
            textcount = textcount + 1
            if textcount == 1 then
                tracktext = tracktext.." "..(text).." "..core:Color(chardb.data[itemid].inv)
            else
                tracktext = tracktext.."   "..(text).." "..core:Color(chardb.data[itemid].inv)
            end
        end
    end
    for k,itemid in pairs (private.cataitems.id) do
        local iconpath = GetItemIcon(itemid)
        local icon = "|T"..iconpath..":12:12:0:0:64:64:4:60:4:60|t"
        local color = private.cataitems.color[k]
        local name = private.cataitems.strings[k]
        local shortname = private.cataitems.short[k]
        local text = core:createText(itemid, iconpath, icon, color, name, shortname, item)
        if text ~= nil then
            textcount = textcount + 1
            if textcount == 1 then
                tracktext = tracktext.." "..(text).." "..core:Color(chardb.data[itemid].inv)
            else
                tracktext = tracktext.."   "..(text).." "..core:Color(chardb.data[itemid].inv)
            end
        end
    end
    for k,itemid in pairs (private.mopitems.id) do
        local iconpath = GetItemIcon(itemid)
        local icon = "|T"..iconpath..":12:12:0:0:64:64:4:60:4:60|t"
        local color = private.mopitems.color[k]
        local name = private.mopitems.strings[k]
        local shortname = private.mopitems.short[k]
        local text = core:createText(itemid, iconpath, icon, color, name, shortname, item)
        if text ~= nil then
            textcount = textcount + 1
            if textcount == 1 then
                tracktext = tracktext.." "..(text).." "..core:Color(chardb.data[itemid].inv)
            else
                tracktext = tracktext.."   "..(text).." "..core:Color(chardb.data[itemid].inv)
            end
        end
    end
    if textcount == 0 then
        tracktext = tracktext.." "..L["Not Tracking"]
    end
   bcm.text = tracktext
end

function core:createText(itemid, iconpath, icon, color, name, shortname, item)
    local item
    local fixeditem = strsub(itemid, 1, -1)
    if chardb.tracking[fixeditem] then
        if chardb.tracking[fixeditem] then
            if not db.showShort then
                item  = "|cff"..color..name.."|r"
            else
                item = "|cff"..color..shortname.."|r"
            end
            if db.showIcons and not db.iconsOnly then
                item = icon..item
            elseif db.showIcons and db.iconsOnly then
                item = icon
            else
                item = item
            end
        end
    end
    return item
end

function core:TooltipDraw()
    local y ,x, justification = tooltip:AddHeader(), 2, {}
    local name = "|cff"..classColors[chardb.class]..self.charName.."|r"
    tooltip:SetCell(y, 2, name)
    tooltip:AddSeparator(2,1,1,1)
    if db.showAlts then
        for char, _ in pairs(charsdb) do
            tinsert(justification, "RIGHT")
            if char ~= self.charName then
                x = x + 1
                tooltip:SetColumnLayout(x, "LEFT", "RIGHT", unpack(justification))
                local name = "|cff".. classColors[charsdb[char].class] .. char .. "|r"
                tooltip:SetCell(y, x, name)
            end
        end
    end
    if db.modules.cmbc then
        for k, itemid in pairs(private.bcitems.id) do
            local iconpath = GetItemIcon(itemid)
            local icon = "|T"..iconpath..":12:12:0:0:64:64:4:60:4:60|t"
            local color = private.bcitems.color[k]
            local name = private.bcitems.strings[k]
            local shortname = private.bcitems.short[k]
            core:createTooltip(itemid, iconpath, icon, color, name, shortname)
        end
    end
    if db.modules.cmlk then
        for k, itemid in pairs(private.lkitems.id) do
            local iconpath = GetItemIcon(itemid)
            local icon = "|T"..iconpath..":12:12:0:0:64:64:4:60:4:60|t"
            local color = private.lkitems.color[k]
            local name = private.lkitems.strings[k]
            local shortname = private.lkitems.short[k]
            core:createTooltip(itemid, iconpath, icon, color, name, shortname)
        end
    end
    if db.modules.cmcata then
        for k, itemid in pairs(private.cataitems.id) do
            local iconpath = GetItemIcon(itemid)
            local icon = "|T"..iconpath..":12:12:0:0:64:64:4:60:4:60|t"
            local color = private.cataitems.color[k]
            local name = private.cataitems.strings[k]
            local shortname = private.cataitems.short[k]
            core:createTooltip(itemid, iconpath, icon, color, name, shortname)
        end
    end
    if db.modules.cmmop then
        for k, itemid in pairs(private.mopitems.id) do
            local iconpath = GetItemIcon(itemid)
            local icon = "|T"..iconpath..":12:12:0:0:64:64:4:60:4:60|t"
            local color = private.mopitems.color[k]
            local name = private.mopitems.strings[k]
            local shortname = private.mopitems.short[k]
            core:createTooltip(itemid, iconpath, icon, color, name, shortname)
        end
    end
end

function core:createTooltip(itemid, iconpath, icon, color, name, shortname)
    local line, x = tooltip:GetLineCount(), 2
    if db.showZero or core:CharHasElement(itemid) then
        line = line + 1
        local item
        if not db.tooltipShowShort then
            item  = "|cff"..color..name.."|r"
        else
            item = "|cff"..color..shortname.."|r"
        end
        if db.tooltipIcons and not db.tooltipIconsOnly then
            item = icon .. " " .. item
        elseif db.tooltipIcons and db.tooltipIconsOnly then
            item = icon
        else
            item = item
        end
        tooltip:SetCell(tooltip:AddLine(), 1, item)
        char = self.charName
        tooltip:SetCell(line, 2, core:CountString(itemid, char))
        x = 2
        if db.showAlts then
            for char, _ in pairs(charsdb) do
                if char ~= self.charName then
                    x = x + 1
                    tooltip:SetCell(line, x, core:CountString(itemid, char))
                end
            end
        end
    end
end